@@ -188,7 +188,6 @@ annotateCoverage(TConfig& c, TSampleLibrary& sampleLib, TCovRecord& ict, TCovera
 {
   typedef typename TSpanMap::value_type::value_type TSpanPair;
   typedef typename TCountMap::value_type::value_type TCountPair;
-  typedef typename TSampleLibrary::value_type TLibraryMap;
   typedef std::vector<uint8_t> TQuality;
   
   // Open file handles
@@ -622,11 +621,12 @@ annotateCoverage(TConfig& c, TSampleLibrary& sampleLib, TCovRecord& ict, TCovera
 	  else outerISize = rec->core.pos + rec->core.l_qseq - rec->core.mpos;
 
 	  // Get the library information
-	  typename TLibraryMap::iterator libIt = _findLib(rec, sampleLib[file_c]);
-	  if (libIt->second.median == 0) continue; // Single-end library or non-valid library
+	  int32_t libIdx = 0;
+	  if ((c.readgroups) && (sampleLib[file_c].size() > 1)) libIdx = _findLib(rec, sampleLib[file_c]);
+	  if (sampleLib[file_c][libIdx].median == 0) continue; // Single-end library or non-valid library
 
 	  // Normal spanning pair
-	  if ((!pairClip) && (getSVType(rec->core) == 2) && (outerISize >= libIt->second.minNormalISize) && (outerISize <= libIt->second.maxNormalISize) && (rec->core.tid==rec->core.mtid)) {
+	  if ((!pairClip) && (getSVType(rec->core) == 2) && (outerISize >= sampleLib[file_c][libIdx].minNormalISize) && (outerISize <= sampleLib[file_c][libIdx].maxNormalISize) && (rec->core.tid==rec->core.mtid)) {
 	    // Take X% of the outerisize as the spanned interval
 	    int32_t spanlen = 0.8 * outerISize;
 	    int32_t pbegin = std::min(rec->core.pos, rec->core.mpos);
@@ -661,7 +661,7 @@ annotateCoverage(TConfig& c, TSampleLibrary& sampleLib, TCovRecord& ict, TCovera
 	  }
 
 	  // Abnormal spanning coverage
-	  if ((getSVType(rec->core) != 2) || (outerISize < libIt->second.minNormalISize) || (outerISize > libIt->second.maxNormalISize) || (rec->core.tid!=rec->core.mtid)) {
+	  if ((getSVType(rec->core) != 2) || (outerISize < sampleLib[file_c][libIdx].minNormalISize) || (outerISize > sampleLib[file_c][libIdx].maxNormalISize) || (rec->core.tid!=rec->core.mtid)) {
 	    // SV type
 	    int32_t svt = _isizeMappingPos(rec, overallMaxISize);
 	    if (svt == -1) continue;
@@ -669,9 +669,9 @@ annotateCoverage(TConfig& c, TSampleLibrary& sampleLib, TCovRecord& ict, TCovera
 	    // Spanning a breakpoint?
 	    bool spanvalid = false;
 	    int32_t pbegin = rec->core.pos;
-	    int32_t pend = std::min(rec->core.pos + libIt->second.maxNormalISize, (int32_t) hdr[file_c]->target_len[refIndex]);
+	    int32_t pend = std::min(rec->core.pos + sampleLib[file_c][libIdx].maxNormalISize, (int32_t) hdr[file_c]->target_len[refIndex]);
 	    if (rec->core.flag & BAM_FREVERSE) {
-	      pbegin = std::max(0, rec->core.pos + rec->core.l_qseq - libIt->second.maxNormalISize);
+	      pbegin = std::max(0, rec->core.pos + rec->core.l_qseq - sampleLib[file_c][libIdx].maxNormalISize);
 	      pend = std::min(rec->core.pos + rec->core.l_qseq, (int32_t) hdr[file_c]->target_len[refIndex]);
 	    }
 	    for(int32_t i = pbegin; i < pend; ++i) {
