@@ -81,11 +81,14 @@ struct Config {
   int32_t indelsize;
   uint32_t graphPruning;
   float flankQuality;
+  bool readgroups;
   bool indels;
   bool hasExcludeFile;
   bool hasVcfFile;
   bool isHaplotagged;
   bool dumpflag;
+  bool svtcmd;
+  std::set<int32_t> svtset;
   DnaScore<int> aliscore;
   boost::filesystem::path outfile;
   boost::filesystem::path vcffile;
@@ -205,7 +208,7 @@ _annotateCoverage(TConfig& c, bam_hdr_t* hdr, TSampleLib& sampleLib, TSVs& svs,
   for(uint32_t file_c = 0; file_c < c.files.size(); ++file_c) {
     countMap[file_c].resize(svs.size());
     for (uint32_t id = 0; id < svs.size(); ++id) {
-      if ((c.indels) && (svSize[id] <= c.indelsize)) {
+      if (svSize[id] <= c.indelsize) {
 	countMap[file_c][id].rc = readCountMap[file_c][id].first;
 	countMap[file_c][id].leftRC = readCountMap[file_c][id + lastId].first;
 	countMap[file_c][id].rightRC = readCountMap[file_c][id + 2*lastId].first;
@@ -257,9 +260,35 @@ inline int dellyRun(TConfigStruct& c) {
 	  int32_t tid = bam_name2id(hdr, chrName.c_str());
 	  if (tid >= 0) {
 	    if (tokIter!=tokens.end()) {
-	      int32_t start = boost::lexical_cast<int32_t>(*tokIter++);
-	      int32_t end = boost::lexical_cast<int32_t>(*tokIter++);
-	      exclg[tid].insert(TIVal::right_open(start, end));
+	      int32_t start = 0;
+	      try {
+		start = boost::lexical_cast<int32_t>(*tokIter++);
+	      } catch (boost::bad_lexical_cast&) {
+		std::cerr << "Exclude file needs to be in tab-delimited format: chr, start, end" << std::endl;
+		std::cerr << "Offending line: " << chrFromFile << std::endl;
+		return 1;
+	      }
+	      if (tokIter!=tokens.end()) {
+		int32_t end = start + 1;
+		try {
+		  end = boost::lexical_cast<int32_t>(*tokIter++);
+		} catch (boost::bad_lexical_cast&) {
+		  std::cerr << "Exclude file needs to be in tab-delimited format: chr, start, end" << std::endl;
+		  std::cerr << "Offending line: " << chrFromFile << std::endl;
+		  return 1;
+		}
+		if (start < end) {
+		  exclg[tid].insert(TIVal::right_open(start, end));
+		} else {
+		  std::cerr << "Exclude file needs to be in tab-delimited format (chr, start, end) and start < end." << std::endl;
+		  std::cerr << "Offending line: " << chrFromFile << std::endl;
+		  return 1;
+		}
+	      } else {
+		std::cerr << "Exclude file needs to be in tab-delimited format: chr, start, end" << std::endl;
+		std::cerr << "Offending line: " << chrFromFile << std::endl;
+		return 1;
+	      }
 	    } else validChr[tid] = false; // Exclude entire chromosome
 	  }
 	}
@@ -287,7 +316,7 @@ inline int dellyRun(TConfigStruct& c) {
   //}
   
   // Create library objects
-  typedef boost::unordered_map<std::string, LibraryInfo> TLibraryMap;
+  typedef std::vector<LibraryInfo> TLibraryMap;
   typedef std::vector<TLibraryMap> TSampleLibrary;
   TSampleLibrary sampleLib(c.files.size());
   if (!getLibraryParams(c, validRegions, sampleLib)) {
@@ -334,10 +363,10 @@ inline int dellyRun(TConfigStruct& c) {
   // Output library statistics
   boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
   std::cout << '[' << boost::posix_time::to_simple_string(now) << "] " << "Library statistics" << std::endl;
-  for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+  for(uint32_t file_c = 0; file_c < c.files.size(); ++file_c) {
     std::cout << "Sample: " << c.sampleName[file_c] << std::endl;
-    for(TLibraryMap::const_iterator libIt = sampleLib[file_c].begin(); libIt != sampleLib[file_c].end(); ++libIt) {
-      std::cout << "RG: ID=" << libIt->first << ",ReadSize=" << libIt->second.rs << ",Median=" << libIt->second.median << ",MAD=" << libIt->second.mad << ",UniqueDiscordantPairs=" << libIt->second.abnormal_pairs << std::endl;
+    for(uint32_t libIdx = 0; libIdx < sampleLib[file_c].size(); ++libIdx) {
+      std::cout << "RG: ID=" << sampleLib[file_c][libIdx].rg << ",ReadSize=" << sampleLib[file_c][libIdx].rs << ",Median=" << sampleLib[file_c][libIdx].median << ",MAD=" << sampleLib[file_c][libIdx].mad << ",UniqueDiscordantPairs=" << sampleLib[file_c][libIdx].abnormal_pairs << std::endl;
     }
   }
   
@@ -357,9 +386,11 @@ int delly(int argc, char **argv) {
   c.isHaplotagged = false;
 
   // Define generic options
+  std::string svtype;
   boost::program_options::options_description generic("Generic options");
   generic.add_options()
     ("help,?", "show help message")
+    ("svtype,t", boost::program_options::value<std::string>(&svtype)->default_value("ALL"), "SV type to compute [DEL, INS, DUP, INV, BND, ALL]")
     ("genome,g", boost::program_options::value<boost::filesystem::path>(&c.genome), "genome fasta file")
     ("exclude,x", boost::program_options::value<boost::filesystem::path>(&c.exclude), "file with regions to exclude")
     ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile)->default_value("sv.bcf"), "SV BCF output file")
@@ -370,7 +401,8 @@ int delly(int argc, char **argv) {
     ("map-qual,q", boost::program_options::value<unsigned short>(&c.minMapQual)->default_value(1), "min. paired-end (PE) mapping quality")
     ("qual-tra,r", boost::program_options::value<unsigned short>(&c.minTraQual)->default_value(20), "min. PE quality for translocation")
     ("mad-cutoff,s", boost::program_options::value<unsigned short>(&c.madCutoff)->default_value(9), "insert size cutoff, median+s*MAD (deletions only)")
-    ("noindels,n", "no small InDel calling")
+    ("indels,i", "activate small InDel calling")
+    ("readgroup,e", "enable read-group aware calling")
     ;
 
   boost::program_options::options_description geno("Genotyping options");
@@ -408,6 +440,29 @@ int delly(int argc, char **argv) {
     return 0;
   }
 
+  // Only one SV type to compute?
+  c.svtcmd = false;
+  if (vm.count("svtype")) {
+    c.svtcmd = true;
+    if (svtype == "DEL") {
+      c.svtset.insert(2);
+    } else if (svtype == "INS") {
+      c.svtset.insert(4);
+    } else if (svtype == "DUP") {
+      c.svtset.insert(3);
+    } else if (svtype == "INV") {
+      c.svtset.insert(0);
+      c.svtset.insert(1);
+    } else if (svtype == "BND") {
+      c.svtset.insert(DELLY_SVT_TRANS + 0);
+      c.svtset.insert(DELLY_SVT_TRANS + 1);
+      c.svtset.insert(DELLY_SVT_TRANS + 2);
+      c.svtset.insert(DELLY_SVT_TRANS + 3);
+    } else {
+      c.svtcmd = false;
+    }
+  }
+  
   // Dump PE and SR support?
   if (vm.count("dump")) c.dumpflag = true;
   else c.dumpflag = false;
@@ -537,9 +592,13 @@ int delly(int argc, char **argv) {
   if (c.minGenoQual<5) c.minGenoQual=5;
 
   // Small InDels?
-  if (vm.count("noindels")) c.indels = false;
-  else c.indels = true;
-  
+  if (vm.count("indels")) c.indels = true;
+  else c.indels = false;
+
+  // Read-group aware calling
+  if (vm.count("readgroup")) c.readgroups = true;
+  else c.readgroups = false;
+
   // Run main program
   c.aliscore = DnaScore<int>(5, -4, -10, -1);
   c.flankQuality = 0.95;
