@@ -618,7 +618,6 @@ namespace torali
   inline void
   shortPE(TConfig const& c, TValidRegion const& validRegions, TVariants& svs, TSampleLib& sampleLib)
   {
-    typedef typename TSampleLib::value_type TLibraryMap;
     typedef typename TValidRegion::value_type TChrIntervals;
 
     // Open file handles
@@ -716,6 +715,7 @@ namespace torali
 	    // Small indel detection using soft clips
 	    if (c.indels) {
 	      for(int32_t svt = 2; svt<5; svt += 2) {
+		if ((c.svtcmd) && (c.svtset.find(svt) == c.svtset.end())) continue;
 		int clipSize = 0;
 		int splitPoint = 0;
 		bool leadingSoftClip = false;
@@ -800,13 +800,15 @@ namespace torali
 	      // SV type
 	      int32_t svt = _isizeMappingPos(rec, overallMaxISize);
 	      if (svt == -1) continue;
+	      if ((c.svtcmd) && (c.svtset.find(svt) == c.svtset.end())) continue;
 
 	      // Library
-	      typename TLibraryMap::iterator libIt = _findLib(rec, sampleLib[file_c]);
-	      if (libIt->second.median == 0) continue; // Single-end library
+	      int32_t libIdx = 0;
+	      if ((c.readgroups) && (sampleLib[file_c].size() > 1)) libIdx = _findLib(rec, sampleLib[file_c]);
+	      if (sampleLib[file_c][libIdx].median == 0) continue; // Single-end library
 
 	      // Check library-specific insert size for deletions
-	      if ((svt == 2) && (libIt->second.maxISizeCutoff > std::abs(rec->core.isize))) continue;
+	      if ((svt == 2) && (sampleLib[file_c][libIdx].maxISizeCutoff > std::abs(rec->core.isize))) continue;
 
 	      // Get or store the mapping quality for the partner
 	      if (_firstPairObs(rec, lastAlignedPosReads)) {
@@ -847,9 +849,9 @@ namespace torali
 		
 #pragma omp critical
 		{
-		  bamRecord[svt].push_back(BamAlignRecord(rec, pairQuality, alignmentLength(rec), alenmate, libIt->second.median, libIt->second.mad, libIt->second.maxNormalISize));
+		  bamRecord[svt].push_back(BamAlignRecord(rec, pairQuality, alignmentLength(rec), alenmate, sampleLib[file_c][libIdx].median, sampleLib[file_c][libIdx].mad, sampleLib[file_c][libIdx].maxNormalISize));
 		}
-		++libIt->second.abnormal_pairs;
+		++sampleLib[file_c][libIdx].abnormal_pairs;
 	      }
 	    }
 	  }
@@ -955,7 +957,7 @@ namespace torali
 	    // Append new edge
 	    TCompEdgeList::iterator compEdgeIt = compEdge.find(compIndex);
 	    if (compEdgeIt->second.size() < c.graphPruning) {
-	      TWeightType weight = (TWeightType) ( std::log((float) abs( abs( (_minCoord(bamItNext->pos, bamItNext->mpos, svt) - minCoord) - (_maxCoord(bamItNext->pos, bamItNext->mpos, svt) - maxCoord) ) - abs(bamIt->Median - bamItNext->Median) ) + 1 ) / std::log(2) );
+	      TWeightType weight = (TWeightType) ( std::log((double) abs( abs( (_minCoord(bamItNext->pos, bamItNext->mpos, svt) - minCoord) - (_maxCoord(bamItNext->pos, bamItNext->mpos, svt) - maxCoord) ) - abs(bamIt->Median - bamItNext->Median)) + 1) / std::log(2) );
 	      compEdgeIt->second.push_back(TEdgeRecord(bamItIndex, bamItIndexNext, weight));
 	    }
 	  }
